function [A,B,C,H,f,iter,fp,La,Lb,Lc]=tuck3abkrep(X,n,m,p,r1,r2,r3,start,conv,A,B,C,H);
% function [A,B,C,H,f,iter,fp,La,Lb,Lc]=tuck3abkrep(X,n,m,p,r1,r2,r3,start,conv,A,B,C,H);
% Tuckals3 via GS, using A&B's shortcut, but using orignal rational start
% NOTE: ALL PRINTINGS SUPPRESSED!!
% 
% input : X   (frontal slices next to each other)
%         conv
%	  n,m,p :order of array
%	  r1,r2,r3
%         start = 0 default
% 	         = 1 random
% 				= 2 start from input 
% work arrays: Y, Z, G
% Output: 
%	  A, B, C: orthonormal component matrices
%	  H = Core
%	  f = loss function value
%	  iter = number of iterations	
%    fp = percentage of fit
%	  La, Lb, Lc matrices which should be diagonal, and if so, contain "intrinsic eigenvalues"
%
% uses ed.m ssq.m permnew.m

% initialize A, B and C

ss=ssq(X);
dys=0;
if start==0 
  % rational starts via eigendecompositions
    [K,L]=ed(X*X');
    A=K(:,1:r1);
    Z=permnew(X,n,m,p);		% yields m x p x n array
    [K,L]=ed(Z*Z');
    B=K(:,1:r2);
    Z=permnew(Z,m,p,n);		% yields p x n x m
    [K,L]=ed(Z*Z');		
    C=K(:,1:r3);
end;    
%disp('  ');
if start==1
 % disp(' random  O R T H O N O R M A L I Z E D  starts');
  if n>=r1,A=orth(rand(n,r1)-.5);else A=orth(rand(r1,r1)-.5);A=A(1:n,:);end;
  if m>=r2,B=orth(rand(m,r2)-.5);else B=orth(rand(r2,r2)-.5);B=B(1:m,:);end;
  if p>=r3,C=orth(rand(p,r3)-.5);else C=orth(rand(r3,r3)-.5);C=C(1:p,:);end;
end;

% Update Core

if start~=2
  Z=permnew(A'*X,r1,m,p);
  Z=permnew(B'*Z,r2,p,r1);
  H=permnew(C'*Z,r3,r1,r2);    
end;

% Evaluate f

if start==2
  Z=B*permnew(A*H,n,r2,r3);
  Z=C*permnew(Z,m,r3,n);
  Z=permnew(Z,p,n,m);	% Z = Xhat, nxmxp
  f=ssq(X-Z);				% use full formula, taking into account possibility of nonoptimal core in start
else
  f=ss-ssq(H);
end;

%fprintf(' Tucker3 function value at start is %12.8f \n',f);
iter=0;
fold=f+2*conv*f;
t0=clock;
while fold-f>f*conv

  iter=iter+1;
  fold=f;

  % update A   (Z=X*C'x B'  ; GS van Z*Z'*A)

  Z=permnew(X,n,m,p);
  Z=permnew(B'*Z,r2,p,n);
  Z=permnew(C'*Z,r3,n,r2);	% yields n x r2 x r3 array
  [A,R]=qr(Z*(Z'*A),0); 
  
  % update B

  Z=permnew(X,n,m,p);
  Z=permnew(Z,m,p,n);
  Z=permnew(C'*Z,r3,n,m);
  Z=permnew(A'*Z,r1,m,r3);	% yields m x r3 x r1 array
  [B,R]=qr(Z*(Z'*B),0); 

  % update C

  Z=permnew(A'*X,r1,m,p);
  Z=permnew(B'*Z,r2,p,r1);	% yields p x r1 x r2 array
  [C,R]=qr(Z*(Z'*C),0); 

  % Update Core

  Z=permnew(A'*X,r1,m,p);
  Z=permnew(B'*Z,r2,p,r1);
  H=permnew(C'*Z,r3,r1,r2);    

  % Evaluate f

  f=ss-ssq(H);
%  if rem(iter,10)==0,fprintf(' Tucker3 function value after iteration %g is %12.8f \n',iter,f);end;
end;
%fprintf(' Final Tucker3 function value after iteration %g is %12.8f \n',iter,f);
% fprintf(' Iterative part took %8.2f seconds \n',etime(clock,t0));
% fprintf('\n\n  Total SSQ  Tuckals3 loss  Tuckals3 fit   perc.\n\n');
ss=ssq(X);
fp=100*(ss-f)/ss;
%fprintf('%10.4f  %10.4f     ',ss,f);
%fprintf('%10.4f     %6.2f  \n\n',ss-f,fp);

% compute "intrinsic eigenvalues"
% eigenvalues for A-mode:
La=H*H';
Y=permnew(H,r1,r2,r3);
Lb=Y*Y';
Y=permnew(Y,r2,r3,r1);
Lc=Y*Y';

