% Tucker3InclBoot.m
% program to interactively carry out all steps in a tucker3 analysis
%
% Input: X = full data set as matrix with frontal slabs next to each other');
%			no missings allowed!
% during program, the matrix X is replaced by Xprep (i.e., the preprocessed version of X)
disp('  ');
disp('  ');
disp('  ');
disp(' WELCOME to the interactive TUCKER3 analysis program');
disp('  ');
disp(' (written by Henk A.L. Kiers, April 2001) ');
disp('  ');
disp('  ');
disp(' The program works fully interactively.');
disp(' Simply answer the questions, or press the Enter key for the default option.');
disp('  ');
disp(' NOTES: ');
disp(' 1. The program does not handle missing data: '); 
disp('    you should make sure that all missing values are replaced by sensible values');
disp(' 2. The data should be available in the matrix X, which is set up as follows:');
disp('    - rows correspond to A-mode entities');
disp('    - columns correspond to combinations of B- and C-mode entities,');
disp('       with B-mode entities nested within C-mode entities');
disp('   For example, a 2 x 2 x 3 array is given as follows:');
disp('     x_1_1_1  x_1_2_1  x_1_1_2  x_1_2_2  x_1_1_3  x_1_2_3');
disp('     x_2_1_1  x_2_2_1  x_2_1_2  x_2_2_2  x_2_1_3  x_2_2_3');
disp(' 3. The program may change matrix X; keep a copy of X !');
disp(' 4. Tucker3.m works in the workspace, and makes (and hence overwrites) many matrices;');
disp('     you are advised to BACKUP the contents of your workspace before analysis with Tucker3.m ');
disp(' 5. You can specify labels for A- B- and C- mode entities (in laba, labb, labc);');
disp('     otherwise standard labels will be made by the program, as laba, labb, labc.');
disp(' 6. All matrix output is tab-delimited, and can hence easily be converted to tables, after copying to Word');
disp(' 7. Faster analysis (for advanced users) can be carried out using splithalft3.m');
disp(' Note: If you are interested in doing a bootstrap analysis, it is assumed that the A-mode is the mode from which you want to resampling');
disp(' If this is not the case, first reorder your data');
disp('  ');
disp(' START of program ');
disp('  ');

conv=1e-6;
cc=0;
while cc==0
  n=input(' Specify the number of A-mode entities ');
  m=input(' Specify the number of B-mode entities ');
  p=input(' Specify the number of C-mode entities ');
  if isempty(n)|isempty(m)|isempty(p),cc=0;disp(' Please specify ALL sizes! ');
  else,cc=1;end;
end;

if exist('X')==0,disp(' Error: Data not available on X');return;end;
Xprep=X;
[rr,cc]=size(Xprep);
if rr~=n | cc~=m*p, disp(' Error: array size does not agree with input data matrix X');
   return;
end;
if exist('laba')==0,laba=num2str([1:n]');aa=length(laba(1,:));laba(:,2:aa+1)=laba;laba(:,1)='A';end
if exist('labb')==0,labb=num2str([1:m]');aa=length(labb(1,:));labb(:,2:aa+1)=labb;labb(:,1)='B';end
if exist('labc')==0,labc=num2str([1:p]');aa=length(labc(1,:));labc(:,2:aa+1)=labc;labc(:,1)='C';end
if length(laba(:,1))~=n,disp(' Error: vector of labels for A (laba) has incorrect size');return;end;
if length(labb(:,1))~=m,disp(' Error: vector of labels for B (labb) has incorrect size');return;end;
if length(labc(:,1))~=p,disp(' Error: vector of labels for C (labc) has incorrect size');return;end;



disp('  ');
cc=input(' To see anova results, specify "1":  ');
if isempty(cc);cc=0;end;
if cc==1
   disp('  ');
   threewayanova(Xprep,n,m,p);
   disp('  ');
end;

% Centering
disp('  ');
disp(' How do you want to CENTER your array ?');
disp(' 1= across A-mode ');
disp(' 2= across B-mode ');
disp(' 3= across C-mode ');
disp(' 12= across A-mode and across B-mode ');
disp(' 13= across A-mode and across C-mode ');
disp(' 23= across B-mode and across C-mode ');
cc=input(' Specify your choice:  ');
if isempty(cc);cc=0;end;
centopt=cc;
disp('  ');
if cc==1 | cc==12 | cc==13
   Xprep=cent3(Xprep,n,m,p,1);
   disp(' Data have been centered across A-mode');
end;
if cc==2 | cc==12 | cc==23
   Xprep=cent3(Xprep,n,m,p,2);
   disp(' Data have been centered across B-mode');
end;
if cc==3 | cc==13 | cc==23
   Xprep=cent3(Xprep,n,m,p,3);
   disp(' Data have been centered across C-mode');
end;

% Normalizing 
disp('  ');
disp(' How do you want to NORMALIZE your array ?');
disp(' 1= within A-mode ');
disp(' 2= within B-mode ');
disp(' 3= within C-mode ');
cc=input(' Specify your choice:  ');
if isempty(cc);cc=0;end;
normopt=cc;
disp('  ');
if cc==1 
   Xprep=norm3(Xprep,n,m,p,1);
   disp(' Data have been normalized within A-mode');
end;
if cc==2 
   Xprep=norm3(Xprep,n,m,p,2);
   disp(' Data have been normalized within B-mode');
end;
if cc==3
   Xprep=norm3(Xprep,n,m,p,3);
   disp(' Data have been normalized within C-mode');
end;

disp('  ');
disp(' NOTE: ');
disp('   The preprocessed data are now available in Xprep');
disp('   Xprep can be used for analyses outside the Tucker3.m program');
disp('  ');

disp('  ');
cc=input(' To see PCA of MEAN, specify "1":  ');
if isempty(cc);cc=0;end;
while cc==1
   [Y,ev,A1,B1,C1,A2,B2,C2]=pcamean(Xprep,n,m,p);
   disp('  ');
   disp(' To study/plot the output you can now temporarily leave the interactive program');
   disp('   to get back, type "return"');
   ccc=input(' If you want to temporarily leave the program, specify "1": '); 
   if isempty(ccc);ccc=0;end;
   if ccc==1,
      disp('  ');
      disp(' You leave the interactive program and can manipulate/study current matrices. ');
      disp(' You may use writescr.m, or use any scatter-plotting routine you like')
      disp(' (e.g., scatter, etc.,or "plotje.m").');
      disp(' If you want to make plots, you may use labels in laba, labb, labc.');
      disp('  ');
      disp(' To return to interactive program, type "return"');
      keyboard;
   end;
   disp('  ');
   ccc=input(' If you want another PCA of mean analysis, specify "1":  ');
   if isempty(ccc);ccc=0;end;
   if ccc~=1,cc=2;end;
end;

disp(' ');
disp(' You are now about to do a Tucker3 analysis.');
disp(' You have to specify the dimensionalities to use.');
disp(' To search these, you are advised to first run PCASUP analyses.');
disp(' (PCASUP analyses are PCA''s of supermatrices with slices of the 3way array next to each other;')
disp('  thus 3 supermatrices are analyed by PCA, and for each we get a component matrix, and eigenvalues;');
disp('  the eigenvalues may give an indication as to the required number of components for each mode.)');
disp('  The results can next be used to find useful dimensionalities by the generalized scree test ');
disp('  (see Timmerman & Kiers, 2000, Kiers & der Kinderen, 2003)');
disp(' ');
cc=input(' If you want to do the PCASUPs for Choosing your dimensionality , specify "1":   ');
if isempty(cc);cc=0;end;
disp('  ');
if cc==1
   disp(' For the generalized scree test it is needed to indicate the maximum number of dimensions for each mode you want to study');
   maxa=input(' Up to how many A-mode components do you want to use?  ');
   maxb=input(' Up to how many B-mode components do you want to use?  ');
   maxc=input(' Up to how many C-mode components do you want to use?  ');
   out=tuckrunsApproxFit(Xprep,n,m,p,maxa,maxb,maxc);
   disp('  ');
   disp('           numbers of components  fit    total number');
   disp('            A       B       C     perc.  of components');
   disp('  ');
   writescr(out,'6.2');
   disp('  ');
   disp(' This info remains available in matrix out ');
   disp('  ');
   disp(' Suggestion based on Convex Hull procedure (Ceulemans & Kiers, 2006) ' );
   disp(' -- programmed by Urbano Lorenzo Seva, 2007 --' );
   DimSelector(out);
   disp(' ');
   disp(' ');
   disp(' Figure 1 now gives fit values of all solutions plotted against P+Q+R');
   disp(' Figure 2 gives fit values of all solutions plotted against number of free parameters');
   tucker3dimensionalityplot;
   disp(' ');
end;


ccc=input(' If you want to temporarily leave the program, specify "1": '); 
if isempty(ccc);ccc=0;end;
if ccc==1,
   disp('  ');
   disp(' You leave the interactive program and can manipulate/study current matrices ');
   disp(' To return to interactive program, type "return"');
   keyboard;
end;

disp('  ');
disp(' You can now do a TUCKER3 ANALYSIS');
disp('  ');
cc=0;
while cc==0
  r1=input(' How many A-mode components do you want to use?  ');
  r2=input(' How many B-mode components do you want to use?  ');
  r3=input(' How many C-mode components do you want to use?  ');
  if isempty(r1)|isempty(r2)|isempty(r3),cc=0;disp(' Please specify ALL numbers of components! ');
  else,cc=1,end;
end;
disp(' ');
disp(' By default, only a rationally started analysis run will be carried out.');
disp(' To decrease the chance of missing the optimal solution, you may use');
disp(' additional, randomly started runs.');
addanal=input(' If you want additional runs, specify how many (e.g., 4):  ');
if isempty(addanal),addanal=0;end;
disp('  ');
[A,B,C,H,f,iter,fp,La,Lb,Lc]=tuck3abk(Xprep,n,m,p,r1,r2,r3,0,conv);
func=ones(1+addanal,1);
func=fp;
for run=1:addanal
   disp('  ');
   fprintf(' Run no. %g \n',run+1);
   disp('  ');
   [Ar,Br,Cr,Hr,fr,iterr,fpr,Lar,Lbr,Lcr]=tuck3abk(Xprep,n,m,p,r1,r2,r3,1,conv);
   func(run+1)=fpr;
   if fpr>1.0001*fp % if fit more than .01% better is found, replace solution
      A=Ar;B=Br;C=Cr;H=Hr;f=fr;iter=iterr;fp=fpr;La=Lar;Lb=Lbr;Lc=Lcr;
   end;
   disp('  ');
end;
disp('  ');
if addanal>=1
   disp('  ');
   disp(' Fit % values from all runs:');
   disp(' ');
   writescr(func','6.2');
   disp('  ');
end;
fprintf(' Tucker3 analysis with %gx%gx%g components gave a fit of %4.2f %% \n',r1,r2,r3,fp); 
disp('  ');
disp('  ');
disp('  ');

disp('  ');
ccc=input(' If you want to temporarily leave the program, specify "1": '); 
if isempty(ccc);ccc=0;end;
if ccc==1,
   disp('  ');
   disp(' You leave the interactive program and can manipulate/study current matrices ');
   disp(' To return to interactive program, type "return"');
   keyboard;
end;
disp('  ');


% Prepare plotting matrices
disp('  ');
disp(' PLOTTING ');
disp('  ');
disp(' No special plotting routines are incorporated, but you can use');
disp(' any scatter-plotting routine you like (e.g., scatter, etc.,or "plotje.m")');
disp('  ');
disp(' If you want to make plots of entities of one mode, you can use:');
disp(' matrices Aplot, Bplot, or Cplot, and use labels in laba, labb, labc.');
Aplot=A*diag(diag(La).^.5);
Bplot=B*diag(diag(Lb).^.5);
Cplot=C*diag(diag(Lc).^.5);
disp('  ');
disp(' If you want to make plots of entities of two modes, ');
disp(' with the third projected in it as axes, you can use the coordinates in:');
disp(' BAplot with those in C,or ACplot with those in B, or CBplot with those in A');
disp(' You will still have to make proper labels for these combined mode plots ');
Aplot=A*diag(diag(La).^.5);
Bplot=B*diag(diag(Lb).^.5);
Cplot=C*diag(diag(Lc).^.5);
Ha=H;Hb=permnew(Ha,r1,r2,r3);Hc=permnew(Hb,r2,r3,r1);
CBplot=kron(C,B)*Ha';
ACplot=kron(A,C)*Hb';
BAplot=kron(B,A)*Hc';
disp('  ');

disp('  ');
ccc=input(' If you want to temporarily leave the program, specify "1": '); 
if isempty(ccc);ccc=0;end;
if ccc==1,
   disp('  ');
   disp(' You leave the interactive program and can manipulate/study current matrices ');
   disp(' To return to interactive program, type "return"');
   keyboard;
end;
disp('  ');

disp(' It is sometimes useful to RENORMALIZE components, e.g., such that')
disp(' the influence of the different A-mode components on the core is equalized');
disp(' This will usually lead to correlated components in that mode, and ');
disp(' to a more simple structure for the components in the OTHER modes, ');
disp(' ');
cc=input(' If you want to renormalize components, specify "1":  ');
if isempty(cc);cc=0;end;
renormmode=0;
disp('  ');
if cc==1
  renormmode=input(' Which mode do you want to renormalize? Enter 1 (=A), 2 (=B) or 3 (=C): ');
  [A,B,C,H]=renormsol(A,B,C,H,renormmode); 
end;

disp('  ');
disp(' Find useful simple structure rotation of core and components');
disp('  ');
disp('  ');
cc=1; 
while cc==1
% varimcocoruns.m
% program to run varimcoco repeatedly with different 
% relative weights for A,B,C
  disp(' ');
  disp(' You can now carry out SIMPLE STRUCTURE rotations with varying weights;');
  disp(' if desired, you can specify a range of different weights for each mode.');
  disp(' To specify a range of values you type, e.g., 1:4 (then the values 1,2,3 and 4 are chosen),');
  disp(' or 1:5:25 (then the values 1,6,11,16 and 21 are chosen),');
  disp(' or [1 2 5 10 100] (then the values 1,2,5,10 and 100 are chosen).');
  disp(' You can also specify a single weight (e.g., just 1).');
  disp(' Analyses will be carried out with all combinations of relative weights.');
  disp('  ');
  aa=input(' Specify (range of) relative weight(s) for A: ');
  bb=input(' Specify (range of) relative weight(s) for B: ');
  cc=input(' Specify (range of) relative weight(s) for C: ');
  disp('  ');
  out=[];
  if isempty(aa);aa=0;end;
  if isempty(bb);bb=0;end;
  if isempty(cc);cc=0;end;
  for wa_rel=aa
   for wb_rel=bb
      for wc_rel=cc
       disp('  ');
       disp('  ');
       fprintf(' Simple structure rotation with relative weights %g %g %g \n',wa_rel,wb_rel,wc_rel);
       disp('  ');
       [AS,BT,CU,K,S,T,U,f,f1,f2a,f2b,f2c,func]=varimcoco(A,B,C,H,wa_rel,wb_rel,wc_rel);
       out=[out;wa_rel wb_rel wc_rel f1 f2a f2b f2c];
     end;
   end;
  end;
  disp('  ');
  disp('         relative weights       simplicity values');
  disp('        A       B       C	   core     A       B       C');
  disp('  ');
  writescr(out,'6.2');   
  disp('  ');
  ccc=input(' If you want to temporarily leave the program, specify "1": '); 
  if isempty(ccc);ccc=0;end;
  if ccc==1,
    disp('  ');
    disp(' You leave the interactive program and can manipulate/study current matrices ');
    disp(' To return to interactive program, type "return"');
    keyboard;
  end;
  ccc=input(' If you want another series of simple structure analyses, specify "1":  ');
  if isempty(ccc);ccc=0;end;
  if ccc~=1,cc=2;end;
end;

disp('  ');
disp(' Give SIMPLE STRUCTURE rotated solution(s) in detail ');
disp('  ');
cc=1;

% make labels for columns of core
str=[' '];
for k=1:r3
   for j=1:r2
      str=[str '       B' int2str(j) 'xC' int2str(k)];
   end;
end;
corelabelstr=str;

while cc==1
  disp(' Use last simple structure rotated solution ? type "return"');
  cc=input(' to specify a different simple structure rotation, specify "1" ');
  if isempty(cc),cc=0;end;
  if cc==1
    disp(' Specify your choice for the preferred simple structure rotation of core and components');
    wa_rel=input(' Specify relative weight for simplicity of A-mode  ');
    wb_rel=input(' Specify relative weight for simplicity of B-mode  ');
    wc_rel=input(' Specify relative weight for simplicity of C-mode  ');
    disp('  ');
    [AS,BT,CU,K,S,T,U,f,f1,f2a,f2b,f2c,func]=varimcoco(A,B,C,H,wa_rel,wb_rel,wc_rel);
  end;
  disp('   ');
  disp(' Backnormalize A,B,C, and H');
  if renormmode==1
      Ds=diag(sum(AS.^2).^.5);
      AS=AS/Ds;
      K=Ds*K;
  end;
  if renormmode==2
      Ds=diag(sum(BT.^2).^.5);
      BT=BT/Ds;
      K=K*kron(eye(r3),Ds);
  end;
  if renormmode==3
      Ds=diag(sum(CU.^2).^.5);
      CU=CU/Ds;
      K=K*kron(Ds,eye(r2));
  end;
  disp('  ');
  disp(' Rotated Solution for A, B and C is in AS, BT and CU');
  disp(' Rotated core in K');
  disp('  ');
  if n<40,disp('  ');disp(' Rotated A (AS) ');disp('  ');writescr(AS,'6.2');end
  disp('  ');disp(' Rotated B (BT) ');disp('  ');writescr(BT,'6.2')
  disp('  ');disp(' Rotated C (CU) ');disp('  ');writescr(CU,'6.2')
  disp('  ');disp(' Rotated core');disp(' ');disp(corelabelstr);disp(' ');writescr(K,'8.2')
  disp('  ');
  cc=0;
  disp(' You can keep present as final solution, or study a different one');
  cc=input(' If you want to study one more solution, specify "1"  '); 
end;

% manual permutation and reflection:
disp('  ');
disp('  You can now manually PERMUTE and REFLECT columns/rows of solution');;
disp('  ');
cc=input(' If you want to reflect/permute columns/rows, specify "1":  ');
if isempty(cc);cc=0;end;
while cc==1
   tau=[];pp=[];
   disp(' ');
   tau=input(' Give vector for reflection of columns of A (e.g., [1 -1 -1 1 ..]) ');
   if isempty(tau),tau=ones(1,r1);end;
   if ssq(tau)~=r1,disp(' incorrect reflection');tau=ones(1,r1);end;
   if ssq(tau.^2-1)~=0,disp(' incorrect reflection');tau=ones(1,r1);end;
   disp(' ');
   pp=input(' Give vector with new order of columns of A   (e.g., [3 1 4 2 ..]) ');
   if isempty(pp),pp=1:r1;end;
   if ssq(size(pp)-size(ones(1,r1)))>0,disp(' incorrect permutation');pp=1:r1;end;
   if ssq(sort(pp')'-[1:r1])>0,disp(' incorrect permutation');pp=1:r1;end;
   AS=AS*diag(tau);K=diag(tau)*K;
   AS=AS(:,pp);K=K(pp,:);
   disp('  ');disp(' Rotated B (BT) ');disp('  ');writescr(BT,'6.2')
   disp('  ');disp(' Rotated C (CU) ');disp('  ');writescr(CU,'6.2')
   disp('  ');disp(' Rotated core');disp(' ');disp(corelabelstr);disp(' ');writescr(K,'8.2')
   tau=[];pp=[];
   disp(' ');
   tau=input(' Give vector for reflection of columns of B (e.g., [1 -1 -1 1 ..]) ');
   if isempty(tau),tau=ones(1,r2);end;
   if ssq(tau)~=r2,disp(' incorrect reflection');tau=ones(1,r2);end;
   if ssq(tau.^2-1)~=0,disp(' incorrect reflection');tau=ones(1,r2);end;
   disp(' ');
   pp=input(' Give vector with new order of columns of B   (e.g., [3 1 4 2 ..]) ');
   if isempty(pp),pp=1:r2;end;
   if ssq(size(pp)-size(ones(1,r2)))>0,disp(' incorrect permutation');pp=1:r2;end;
   if ssq(sort(pp')'-[1:r2])>0,disp(' incorrect permutation');pp=1:r2;end;
   K=permnew(K,r1,r2,r3);
   BT=BT*diag(tau);K=diag(tau)*K;
   BT=BT(:,pp);K=K(pp,:);
   K=permnew(K,r2,r3,r1);
   K=permnew(K,r3,r1,r2);
   disp('  ');
   disp('  ');disp(' Rotated C (CU) ');disp('  ');writescr(CU,'6.2')
   disp('  ');disp(' Rotated core');disp(' ');disp(corelabelstr);disp(' ');writescr(K,'8.2')
   tau=[];pp=[];
   disp(' ');
   tau=input(' Give vector for reflection of columns of C (e.g., [1 -1 -1 1 ..]) ');
   if isempty(tau),tau=ones(1,r3);end;
   if ssq(tau)~=r3,disp(' incorrect reflection');tau=ones(1,r3);end;
   if ssq(tau.^2-1)~=0,disp(' incorrect reflection');tau=ones(1,r3);end;
   disp(' ');
   pp=input(' Give vector with new order of columns of C  (e.g., [3 1 4 2 ..]) ');
   if isempty(pp),pp=1:r3;end;
   if ssq(size(pp)-size(ones(1,r3)))>0,disp(' incorrect permutation');pp=1:r3;end;
   if ssq(sort(pp')'-[1:r3])>0,disp(' incorrect permutation');pp=1:r3;end;
   K=permnew(K,r1,r2,r3);
   K=permnew(K,r2,r3,r1);
   CU=CU*diag(tau);K=diag(tau)*K;
   CU=CU(:,pp);K=K(pp,:);
   K=permnew(K,r3,r1,r2);
   disp('  ');disp(' Rotated C (CU) ');disp('  ');writescr(CU,'6.2')
   disp('  ');disp(' Rotated core');disp(' ');disp(corelabelstr);disp(' ');writescr(K,'8.2')
   disp(' ');
   disp(' ');
   cc=input(' If you want to further reflect/permute columns/rows, specify "1":  ');
   if isempty(cc);cc=0;end;
end;
if n<40,disp('  ');disp(' Rotated A (AS) ');disp('  ');writescr(AS,'6.2',laba);end
disp('  ');disp(' Rotated B (BT) ');disp('  ');writescr(BT,'6.2',labb);
disp('  ');disp(' Rotated C (CU) ');disp('  ');writescr(CU,'6.2',labc);
disp('  ');disp(' Rotated core');disp(' ');disp(corelabelstr);disp(' ');writescr(K,'8.2')
disp('  ');
disp('  ');
disp(' You can produce a JOINT PLOT now');
disp(' In joint plots, one mode is fixed, while the entities of the other two are plotted');
cc=input(' If you want to produce a joint plot for the current solution, specify "1": ');
if isempty(cc);cc=0;end;
while cc==1
   fixmode=input(' Which mode do you want to keep fixed ("1", "2" or "3")?  ');
   if fixmode==1 |fixmode==2 |fixmode==3 
      fixunit=input(' For which component do you want to see the joint plot? ');
      fs=7;			% fontsize fixed to 7
      fit=jointplotgen(K,AS,BT,CU,fixmode,fixunit,laba,labb,labc,fs);
      disp(' ');
      fprintf(' The plot displays %4.2f %% of the information represented by the component of your choice\n',fit);
      disp(' ');
   else 
      disp(' fixed mode specified wrongly');
   end;
   disp('  ');
   cc=0;
   cc=input(' Do you want to produce another joint plot? If so, type "1": ');
   disp('  ');
end;
disp('  ');
disp('  ');

ccc=1;
while ccc==1
  disp(' If you want to carry out a BOOTSTRAP procedure for ');
  ccc=input(' computing confidence intervals for the current solution, specify "1": ');
  if isempty(ccc);ccc=0;end;
  if ccc==1
     disp(' It is assumed that the A-mode is the mode from which you want to resampling');
     disp(' If this is not the case, first reorder your data');
     A=AS;B=BT;C=CU;G=K;
     disp(' Procedure uses optimal matching of bootstrap solutions to original solution');
     optimalmatch=1;    % for specialized use, one may change this into 0, 
                        % to get orthogonal rotations
     bootstraptucker3
     % make labels for columns of core
     str=[''];
     for k=1:r3
       for j=1:r2
         str=[str '         B' int2str(j) 'xC' int2str(k) '  '];
       end;
     end;
     corelabelstr2=str;
     % make labels for intervals of columns of B and C
     strB=[' '];
     strC=[' '];
       for j=1:r2
         strB=[strB '         comp. ' int2str(j)];
       end;
       for k=1:r3
         strC=[strC '         comp. ' int2str(k)];
       end;
    
     disp('  ');disp(' Bootstrap confidence intervals for fit percentage ');disp('  ');writescr([lo_fp up_fp],'6.2')
     disp('  ');disp(' Bootstrap confidence intervals for B, per component next to each other ');disp('  ');disp(strB);disp('  ');writescr(Bint,'6.2',labb);
     disp('  ');disp(' Bootstrap confidence intervals for C, per component next to each other ');disp('  ');disp(strC);disp('  ');writescr(Cint,'6.2',labc);
     disp('  ');disp(' Bootstrap confidence intervals for Core ');disp('  ');disp(corelabelstr2);disp('  ');writescr(Gint,'6.2')
     disp('  ');
     ccc=0;
  end;
end;
disp('  ');

ccc=1;
while ccc==1
  ccc=input(' If you want to carry out a STABILITY CHECK on current or different solution, specify "1": ');
  if isempty(ccc);ccc=0;end;
  if ccc==1
     splithalft3
  end;
end;
disp('  ');

disp('  ');
disp(' Press any key to conclude analysis with FITPARTITIONING');
pause;
disp('  ');
% computation of residuals (on Res) and partitioning of fit
disp(' ');
disp(' Contribution to fit (in %s) for all combinations of components');
disp(' These contributions can simply be added to get aggregate contributions of in case all components are orthogonal');
disp(' If this is not the case, aggregate fit contributions are given separately, below');
ssqX=ssq(Xprep);
disp('  ');disp(corelabelstr);disp(' ');writescr((K.^2)*100/ssqX,'8.2');
[fitA,fitB,fitC,BCcontr,ACcontr,ABcontr]=t3fitpartitioning(Xprep,n,m,p,AS,BT,CU,K,renormmode);
if n<40
   disp(' ');
   disp(' Relative fit of A-mode entities (in %)');
   disp(' ');
   writescr(fitA,'6.1',laba);
end;
disp(' ');
disp(' Relative fit of B-mode entities (in %)');
disp(' ');
writescr(fitB,'6.1',labb);
disp(' ');
disp(' Relative fit of C-mode entities (in %)');
disp(' ');
writescr(fitC,'6.1',labc);

disp(' ');
disp(' NOTE: Residuals on "Res", can be used for further analyses');
disp(' ');
disp(' Full rotated solution can be found in AS,BT,CU,K');
